<?php

use EventPortal\Domain\Address\Handlers\CreateAddress;
use EventPortal\Domain\Address\Handlers\UpdateAddress;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Display;
use EventPortal\Domain\Component\Dropdown\Select;
use EventPortal\Domain\Component\Input\Text;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Component\Selection\Checkbox;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Handlers\CreateModel;
use EventPortal\Domain\Handlers\UpdateModel;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Enums\InertiaMethod;

$form = [
    Section::make('Key Details')->fields([
        Text::make('Name')
            ->placeholder('The Heath Hotel')
            ->rules(['required']),
        Text::make('Website')
            ->placeholder('https://theheathhotel.co.uk')
            ->rules(['required', 'url']),
        Select::make('Star Rating')
            ->placeholder('Select a star rating')
            ->options([
                Option::make(1),
                Option::make(2),
                Option::make(3),
                Option::make(4),
                Option::make(5),
            ]),
        Checkbox::make('Has a `Green Tourism` accreditation')
            ->name('has_green_tourism_accreditation'),
    ]),
    Section::make('Address')->fields([
        Text::make('First Line')
            ->name('line_1')
            ->placeholder('The Heath Hotel')
            ->scopeTo('address')
            ->rules(['required']),
        Text::make('Second Line')
            ->name('line_2')
            ->placeholder('Sandy Lane')
            ->scopeTo('address'),
        Text::make('Third Line')
            ->name('line_3')
            ->scopeTo('address'),
        Text::make('Town/City')
            ->name('city')
            ->placeholder('London')
            ->scopeTo('address')
            ->rules(['required']),
        Text::make('Area')
            ->placeholder('Greater London')
            ->scopeTo('address'),
        Text::make('Country')
            ->scopeTo('address')
            ->placeholder('England')
            ->rules(['required']),
        Text::make('Area Code')
            ->placeholder('EC1M 4TU')
            ->scopeTo('address')
            ->rules(['required']),
    ]),
];

Form::make('venue.create')
    ->addSaveHandler(new CreateModel)
    ->addSaveHandler(new CreateAddress)
    ->model(new Venue)
    ->addFields($form)
    ->addActions(function () {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.venue.index'),
            Button::make('Add Venue')
                ->method(InertiaMethod::POST())
                ->route('admin.venue.store'),
        ];
    });

Form::make('venue.edit')
    ->addSaveHandler(new UpdateModel)
    ->addSaveHandler(new UpdateAddress)
    ->addFields($form)
    ->addActions(function (Venue $venue) {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.venue.index'),
            Button::make('Update Venue')
                ->method(InertiaMethod::POST())
                ->route('admin.venue.update', ['venue' => $venue]),
        ];
    });

Form::make('venue.show')
    ->addFields(function () {
        return [
            Section::make('Key Details')->fields([
                Display::make('Name'),
                Display::make('Website'),
            ]),

            Section::make('Address')->fields([
                Display::make('First Line')->name('line_1')->scopeTo('address'),
                Display::make('Second Line')->name('line_2')->scopeTo('address'),
                Display::make('Third Line')->name('line_3')->scopeTo('address'),
                Display::make('City')->scopeTo('address'),
                Display::make('Area')->scopeTo('address'),
                Display::make('Country')->scopeTo('address'),
                Display::make('Area Code')->scopeTo('address'),
            ]),
        ];
    })
    ->addActions(function (Venue $venue) {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.venue.index'),
            Button::make('Edit Venue')
                ->method(InertiaMethod::VISIT())
                ->route('admin.venue.edit', ['venue' => $venue]),
        ];
    });
