<?php

namespace EventPortal\Domain\Util\Notification;

use EventPortal\Domain\Util\Notification\Models\EventNotification;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Str;

class SystemNotification extends Notification implements ShouldQueue
{
    use Queueable;

    public $subject;

    public $data;

    public $key;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public function __construct(string $key, ?string $subject, ?array $data)
    {
        $this->key = $key; // TEMP
        $this->subject = $subject;
        $this->data = $data;
    }

    /**
     * Get the notification's delivery channels.
     * Could use $notifiable for user preferences (email/sms/database) in future
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable): array
    {
        if ($this->isForAnEvent()) {
            EventNotification::create([
                'delegate_id' => $notifiable->delegate->id,
                'event_id' => $this->data['event']->id,
                'method' => 'email',
                'key' => $this->key,
                'status' => 'queued',
            ]);
        }

        return [
            'mail',
        ];
    }

    /**
     * Determine which queues should be used for each notification channel.
     *
     * @return array
     */
    public function viaQueues(): array
    {
        return [
            'mail' => 'email',
        ];
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     */
    public function toMail($notifiable): MailMessage
    {
        $htmlView = "EventPortal::email.{$this->key}";
        $plainTextView = "EventPortal::email.{$this->key}-plain";

        return (new MailMessage())
            ->subject($this->subject)
            ->view(
                [$htmlView, $plainTextView],
                [
                    'data' => $this->data,
                    'recipient' => $notifiable,
                ]
            );
    }

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     */
    // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
    public function toArray($notifiable): array
    {
        // phpcs:enable
        return [
            'data' => $this->data,
        ];
    }

    private function isForAnEvent(): bool
    {
        return Str::startsWith($this->key, 'event.');
    }
}
